;================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;================================

ORG 4C46			;19526

@Keyboard=02BB			;Keyboard scanning subroutine
@Decode=07BD			;Keyboard decode subroutine
@Laddr=09D8			;Line address subroutine
@Vars=4010
@DFile=400C			;Address of display file
@Spare1=4021			;16417 Spare 1, 1 Byte
@NxtLine=4029			;Next program line to be executed
@Seed=4032			;16 bit parameter passing via RAND
@LastK=4025			;Keyboard debounce value
@MainForm=4777			;Stored mainform address

;================================
; Jump Table
;================================
	JP KeyLoop		;19526, Key loop and sorter
	JR LoadScr		;19529, Load main form
	JR SaveScr		;19531, Save main form
	JR Release		;19533, Test for keypress
	JP PrntPos		;19535,	Calculate Print Position
	JP FileDrop		;19538, File dropdown menu
	JP ScrDrop		;19541, Screens dropdown menu
	JP HelpDrop		;19544, Help dropdown menu
        JP ChromaTest		;19547, Test for Chroma
@TPage
	00			;Screens dropdown page number
;================================
; Load screen
;================================
@LoadScr
	PUSH HL
	LD HL,MainForm		;Set source
	LD DE,(DFile)		;Set destination
	JR +7
;================================
; Save screen
;================================
@SaveScr
	PUSH HL
	LD HL,(DFile)		;Set source
	LD DE,MainForm		;Set destination
	LD BC,0319		;BC=byte count
	LDIR			;Copy source to desination
	POP HL
	RET			;Return 0
;================================
; Wait for key release
;================================
@Release
	LD A,(LastK)		;Test for keypress
	CP FF			;Keypressed?
	JR NZ Release		;Yes, loop back
	RET
;================================
; Print dropdown menu
;================================
@DropDown
	CALL SaveScr		;Save main screen
	LD HL,(NxtLine)
@DropMenu
	LD BC,0005		;Skip REM header
	ADD HL,BC		;Start of data
	CALL Release		;Wait for key release
	LD B,(HL)		;Get Ycoord
	INC HL
	LD C,(HL)		;Get Xcoord
	INC HL			;Start of print data
	PUSH HL			;PUSH datapointer
	CALL PrntPos		;Get print position of the NW corner of dropdown
	POP BC			;POP Datapointer into BC
@DropY
	LD (Drop+1),HL		;Save start of row addr
@DropLoop
	LD A,(BC)		;Get chr to print
	CP 76			;EO REM?
	RET Z			;Return printing done
	CP 0C			;EOL marker ()?
	JR Z Drop		;Jump if EOL
	LD (HL),A		;Print A
	JR +7
@Drop
	LD HL,0000		;Get start of row addr
	ADD HL,DE		;Next row addr
	INC BC			;Increase data pointer
	JR DropY		;Start of row addr
	INC HL			;Increase print position
	INC BC			;Increase data pointer
	JR DropLoop
;================================
; File menu dropdown
;================================
@FileDrop
	CALL DropDown		;Print file dropdown menu
	CALL KeyScan		;Start Keyboard loop
	JR Z -5			;Loop until keypressed
	LD HL,FileKeys		;Valid keys table
	LD BC,000E 		;Byte count
	CPIR 			;Locate key
	JR NZ +1		;Jump if no match
	INC B			;B=Hi byte of line number
@SetLine
	LD C,(HL)
	LD H,B
	LD L,C
	CALL Release
	CALL Laddr
	LD (NxtLine),HL
	JP Loadscr
;================================
@FileKeys			;Key code/lo byte of line number
	"P" 90			;400 Play Slideshow
	"B" 40			;320 Back
	"N" 4A			;330 Next
	"D" 54			;340 Duration
	"S" 5E			;350 Shuffle
	"M" 68			;360 Mono/Chroma
	"C" 72			;370 Splash Screen
	5A			;EOTable marker=GOTO 90
;================================
; Screens dropdown
;================================
@ScrDrop
	CALL DropDown		;Print Screens dropdown
	LD H,01 		;Hi byte of REM line
	LD A,(TPage)		;Get current page number
	ADD A,58		;Adjust page number 
	LD L,A			;Lo byte of REM line, HL=501,2,3 or 4
	CALL LAddr		;Get line address
	CALL DropMenu		;Print dropdown
	CALL KeyScan		;Start Keyboard loop
	JR Z -5			;Loop until keypressed
	CP "A"			;Key<"A"?
	JR NC TestW		;Jump if key<"A"
@WrongKey
	CALL LoadScr		;Restore mainform
	RET			;Return to BASIC
@TestW
	CP "W"			;Key>="W"?
	JR NC WrongKey		;Jump if key>"V"
	SUB 25 			;A=0 to 22
	LD D,A 			;Save A			
	CALL Release		;Wait for key release
	LD A,D 			;Restore A
	CP 16			;A="V"
	JR NZ +14		;Jump if not "V"
	LD HL,(Vars)		;Locate page number
	LD BC,000E 
	SBC HL,BC 
	LD A,(HL) 		;Get page number
	LD (TPage),A 		;Set page number
	JR ScrDrop+3		;Print next page
	LD A,(Tpage) 		;Get page number
	SUB 9D			;A=0,1,2,3,4...
	AND A			;A=0?
	LD B,A			;Set DJNZ counter
	LD A,H			;A=0, Z flag not changed
	JR Z +4			;Jump if A was 0
	ADD A,15		;Add offset to A (A=A+21)
	DJNZ -4			;Loop until B=0
	ADD A,D			;Add item number to offset
	LD C,A 			;BC=Screen number
	PUSH BC 		;PUSH Screen number
	CALL LoadScr		;Restore screen
	POP BC 			;POP Screen number
	RET			;Return Screen number to BASIC
;================================
; Help menu dropdown
;================================
@HelpDrop
	CALL DropDown		;Print help dropdown
	CALL KeyScan		;Start Keyboard loop
	JR Z -5			;Loop until keypressed
	CP "C"			;"C"olor Test
	JR NZ KeyH
	LD HL,+800		;GOTO 800
	JP SetLine+3
@KeyH
	CP "H"			;"H"ot keys
	JR NZ KeyK
	LD HL,+630		;GOTO 630
	JP SetLine+3
@KeyK
	CP "K"			;"K"ey Guide
	JR NZ +2
	JR KeyGuide
	CP "A"			;"A"bout
	JR Z About
@GOTO90
	LD HL,+90		;GOTO 90
	JP SetLine+6
;================================
; Key Guide
;================================
@KeyGuide
	LD HL,+615		;REM 615
	JR +3
;================================
; About
;================================
@About
	LD HL,+620		;REM 620
	CALL LAddr
	LD (NxtLine),HL
	CALL LoadScr		;Reload main screen
	CALL DropDown+3		;Print REM 715 or 720
	CALL KeyScan		;Start Keyboard loop
	JR Z -5			;Loop until keypressed
	CALL Release		;Wait for key release
	CALL LoadScr		;Reload main screen
	JR GOTO90
;================================
; Print Position
;  B=Ycoord
;  C=Xcoord
;  BC unchange
;  On exit HL=Print Position address
;================================
@PrntPos
	PUSH BC			;PUSH YXcoords
	LD HL,0000		;Set accumulator to 0
	LD DE,0021		;Bytes/row
	LD A,B
	AND A			;Row=0?
	JR Z +3			;Yes, move on
	ADD HL,DE		;HL=33*row value
	DJNZ -3
	ADD HL,BC		;HL=33*row value+column offset
	LD BC,(DFile)		;Get DFile
	INC BC			;Skip 1st N/L chr
	ADD HL,BC		;HL=print position
	POP BC			;POP YXcoords
	RET	
;================================
; Test for keypress.
; On exit...
;   A=chr code
;================================
@KeyScan
	CALL Keyboard		;Call keyboard scanning routine
	LD B,H 			;BC=Raw Key values
	LD C,L 
	LD D,C			;Test for keypress, D=255 if no key press
	INC D 			;0?
	RET Z
	CALL Decode		;Call keyboard decode routine, BC to ZX81 codes
	INC A			;Set Z flag
	LD A,(HL)
	RET			;Return chr code to BASIC
;================================
; Key loop
;================================
@KeyLoop
	CALL TrackName		;Track name display
@KeyTest
	CALL KeyScan		;Get key press
	JR Z KeyLoop		;No, loop until key pressed
	CP "F"			;"F"ile dropdown?
	JR NZ +4
	LD BC,+250
	RET			;GOTO 250
	CP "G"			;"G"allery dropdown?
	JR NZ +4
	LD BC,+500
	RET			;GOTO 500
	CP "H"			;"H"elp dropdown?
	JR NZ +4
	LD BC,+600
	RET			;GOTO 600
	CP "X"			;E"x"it key?
	JR NZ +4
	LD BC,+700
	RET			;GOTO 700
@NextSample
	LD HL,FileKeys		;Hot keys table
	LD BC,0012 		;Byte count
	CPIR 			;Locate key
	JR NZ  Keyloop		;Loop if none found
	CP "B"			;"B"ack?
	JR Z +2
	CP "N"			;"N"ext
	CALL NZ Release		;Wait for key release except for "B" & "N"
	LD B,01 		;Hi byte line number
	LD C,(HL) 		;Lo byte line number
	RET			;GOTO BC
;================================
; Track Name Display
;================================
@TrackName
	LD HL,(DFile)		;Get DFile
	LD DE,00D0		;Offset to track name
	ADD HL,DE		;HL=Track name location
	LD D,H			;DE=Track name location
	LD E,L
	LD B,0C			;Byte count
;NameLoop
	LD A,(HL)		;Get chr at HL
	ADD A,80		;Add 128
	LD (HL),A		;Set new chr
	INC HL			;Next chr
	LD A,(LastK)		;Test for keypress
	CP FF			;Keypressed?
	JR Z +9			;Jump to delay if no keypress
	EX DE,HL		;Set HL to track name location
	LD B,0C			;Byte count
	RES 7,(HL)		;Reset track name
	INC HL
	DJNZ -5
	RET
;Delay
	PUSH BC			;PUSH byte count
	LD BC,0A01		;Delay loop
	DEC BC 
	LD A,B 
	OR C 
	JR NZ -5
	POP BC			;POP byte count
	DJNZ -33		;Jump to NameLoop until B=0
	RET
;================================
; Chroma test
;================================
@ChromaTest
        LD BC,7FEF		;Chroma color input port.
	IN A,(C) 
	BIT 5,A
	LD BC,0000
	RET NZ			;IF NOT Chroma RETURN 0000
	INC C
	RET			;IF Chroma RETURN 0001
