;================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;================================

ORG 4A98			;19096

@PushReg=0220			;Push reg then start Display-1
@Disp3=0292			;DISPLAY BLANK LINES routine 
@Video01=0281			;Video-1 routine
@PopReg=02A4			;Pop reg then return control to program
@Keyboard=02BB			;KEYBOARD SCANNING subroutine
@Decode=07BD			;KEYBOARD DECODE subroutine
@Fast=0F23			;FAST subroutine
@Slow=0F2B			;SLOW subroutine
@Break=0F46			;Break-1 routine
@DFile=400C			;Address of display file
@LastK=4025			;Keyboard debounce value
@Frames=4034			;SYS VAR: Frames counter
@HiRAM=8000
@HRDFile=8100			;Hires DFile
@SCRDump=9D00
;================================
; WRX Video driver by Wilf Rigter
; WRX16K - 1996
; Annotations by Wilf Rigter
;================================
@WRX
	LD IX,HR		;Simple start of the hres mode
	RET
@LBuf
	LD R,A			;Load HRDFile address LSB
	80 80 80 80		;32 NOPs = 256 pixels
	80 80 80 80 		;80=Black on white display
	80 80 80 80 		;80=White on black display
	80 80 80 80 
	80 80 80 80 
	80 80 80 80 
	80 80 80 80 
	80 80 80 80 
	RET NZ			;Always returns
@HR    
	LD B,06			;Delay
@HR0
	DJNZ HR0
	NOP			;More delay fine tuning
	NOP
	NOP
	NOP
	NOP
	DEC B
@HDAddr
	LD HL,HRDFile		;HRDFile
	LD DE,20		;32 bytes per line
	LD B,C0			;192 lines per hires screen
@HR1
	LD A,H			;Get HRDFile address MSB
	LD I,A			;Load MSB into I register
	LD A,L			;Get HRDFile address LSB
	CALL CA9D 		;CALL Lbuf+8000
	ADD HL,DE		;Next line
	DEC B			;Dec line counter
	JP NZ HR1		;Last line
@HR2   
	CALL Disp3		;Return to application program
	CALL PushReg		;Extra register PUSH and VSYNC
               			;This code segment is optional
@Brake
	CALL Break		;Check break key
	LD A,1E			;Restore pattern table pointer
	LD I,A
	JR NC Stop		;Skip the HR vector load if BREAK
	LD IX,HR		;Load the HR vector
@Stop
	JP PopReg		;Return to application program
;================================
; Jump table
;================================
	JR WRX			;19188
	JR LoRes		;19190
	JR HRCls		;19192
	JR Invert		;19194
	JP Display		;19196
	JP Slide		;19199
	00 00 00		;19202
	JP SCRDisplay		;19205
        JP ScrSlide		;19208
;================================
; Restore lores display
;================================
@Lores
	HALT			;Wait for an interrupt - used to sync the changeover!
	LD A,(Frames) 		;Wait for the next new tv frame
	LD C,A
@Sync
	LD A,(Frames)
	CP C
	JR Z Sync
	LD A,1E		 	;Reset the "I" Register
	LD I,A		 	;to the ROM default (30 decimal)
	LD IX,Video01	 	;Pointer to the ROM display routine set!
	LD BC,0000
	RET
;================================
; WRX invert screen
;================================
@Invert
	LD HL,Lbuf+2		;Set source
	LD BC,2000		;Byte count, 32
	LD A,(HL) 		;Invert the value by adding 128
	ADD A,80
	LD (HL),A		;Set the inverted value 
	INC HL
	DJNZ Invert+6		;Repeat 32x
	RET			;Return 0 to BASIC
;================================
; Clear the HRDFile
;================================
@HRCls
	LD HL,HRDFile		;Set source
	LD (HL),00		;Set source to 0
	LD DE,HRDFile+1		;Set destination
	LD BC,17FF		;Byte count, HRDFile size-1
	LDIR			;Clear the HRDfile
	RET			;RET 0 to BASIC
;================================
; Wait for key release
;================================
@Release
	CALL Keyboard		;Call ROM scan keyboard routine.
	INC L			;Check if user's finger on key.
	JR NZ Release		;If so, go back and check again.
@Ret0
	LD BC,0000
	RET			;RET to BASIC, BC=0000
;================================
; Test for keypress.
; On exit...
;  A=key chr code
;================================
@KeyScan
	CALL Keyboard		;Call keyboard scanning routine
	LD B,H 			;BC=Raw Key values
	LD C,L 
	LD D,C			;Test for keypress 
	INC D 
	LD A,01 
	RET Z 			;No keypress
	CALL Decode		;Call keyboard decode routine, BC to ZX81 codes
	LD A,(HL)		;A=CHR CODE
	RET
;================================
; MC INKEY$ function
; On exit...
;  A=key chr code
; BC=key chr code
;================================
@Inkey
	CALL Release		;Wait for no key press
	CALL KeyScan		;Get keypress
	CP 01			;No key?
	JR Z Inkey+3		;Yes, loop until a keypressed
	LD B,00			;Return keycode
	LD C,A
	RET
;=========================================
; Flash 
;=========================================
@Flash
	CALL Release
	CALL Invert
	CALL Dlay
	CALL KeyScan		;Get keypress
	CP 01			;No key?
	JR Z Flash+3		;Yes, loop until a keypressed
	RET
;================================
; WRX vertical flip
;================================
@Flip
	LD HL,HRDFile		;Start of HRDFile
	LD (HiRAM),HL		;Save top row addr
	LD BC,17E0		;BC=191 rows * 32 Bytes/row
	ADD HL,BC		;HL=Address of bottom row.
	LD B,60			;Rows to process counter
@FlipLoop
	PUSH BC			;PUSH counter
	LD (HiRAM+2),HL		;Save bottom row addr
	LD HL,(HiRAM)		;Transfer
	LD DE,HiRAM+4		;top row data
	LD BC,0020		;to
	LDIR			;storage area
	LD HL,(HiRAM)		;Transfer
	EX DE,HL		;bottom row
	LD HL,(HiRAM+2)		;data
	LD C,20			;to
	LDIR			;top row address
	LD (HiRAM),DE		;Save top row address
	LD HL,HiRAM+4		;Transfer
	LD DE,(HiRAM+2)		;top row data
	LD C,20			;to
	LDIR			;bottom row address
	LD C,20			;Calculate 
	LD HL,(HiRAM+2)		;address
	XOR A			;of
	SBC HL,BC		;next bottom row
	POP BC			;POP row counter
	DJNZ FlipLoop		;Repeat if NZ
	RET
;=========================================
;Delay loop
;=========================================
@Dlay
	LD B,10
	LD C,B 
	INC C 
@DLoop
	DEC BC 
	LD A,B 
	OR C 
	JR NZ DLoop
	RET
;================================
; .bmp display
;================================
@Display
	CALL Flip		;Flip the image
	CALL Slow
	CALL WRX		;Start hires display
@KeyLoop
	CALL Release		;Wait for no keypress
	CALL Inkey		;Wait for keypress
	CP "I"			;"I"nvert
	JR NZ +5		;No, move on
	CALL Invert		;Yes, invert screen
	JR KeyLoop
	CP "V"			;"V"ertical flip?
	JR NZ +5		;No, move on
	CALL Flip		;Vertical flip
	JR KeyLoop
	CP "F"			;"F"lash cycle
	JR NZ +5
	CALL Flash
	JR KeyLoop		;Loop until
	CALL Release		;Wait for no keypress
	JP LoRes		;Yes, start lores mode and return to BASIC
;================================
; .bmp Slide
;================================
@Slide
	CALL Flip		;Flip the image
	CALL Slow
	JP WRX			;Start hires display
;===============================
; .scr display
;===============================
@ScrDisplay
	CALL SCRtoZX
	JP Display+3
;===============================
; .scr Slideshow
;===============================
@ScrSlide
	CALL SCRtoZX
	JP Slide+3
;===============================
; Convert Spectrum .scr file to
; ZX81 format.
;
; LOAD S$+".SCR;40192"
;===============================
@SCRtoZX
	LD B,03			;Counter, 3 thirds to process
	LD HL,SCRDump		;Source, SCR data
	LD DE,HRDFile		;Destination
@SCRLoop1
	PUSH BC			;PUSH thirds counter
	PUSH HL			;PUSH source address
	LD A,08			;Counter, 8 chr rows/SCR third
@SCRLoop2
	PUSH AF			;PUSH Chr rows counter
	LD A,08			;Counter, 8 pixel rows/chr row
	PUSH HL			;PUSH SCR dump address
@SCRLoop3
	LD (CopySCR+1),HL	;Set source
@CopySCR
	LD HL,0000		;Source, SCR dump data
	LD BC,0020		;Byte count, 32 bytes/row
	LDIR			;Copy row to HRDFile
	LD BC,00E0		;Offset to next pixel row
	ADD HL,BC		;Next pixel row source
	DEC A			;Decrease pixel row counter
	JR NZ SCRLoop3		;Do 8 pixel rows
	POP HL			;POP SCR dump address
	LD C,20			;32 bytes/row
	ADD HL,BC		;Next pixel row in chr row
	POP AF			;Chr row counter
	DEC A			;Decrease chr row counter
	JR NZ SCRLoop2		;Do 8 chr rows
	POP HL			;POP source address
	LD BC,0800		;Offset to next third, 2048 bytes
	ADD HL,BC		;Set Source address
	POP BC			;POP thirds done counter
	DJNZ SCRLoop1		;Loop until B=0
	RET
	NOP